import aiohttp
import datetime
from .http import Http

n = datetime.datetime.now()
now = f'{n.year}{n.month}{n.day}'


class Client:
    def __init__(self, KEY='', Type='json', pIndex=str(1), pSize=str(100)):
        self.http = Http(KEY, Type, pIndex, pSize)


    async def schoolInfo(self, SD_SCHUL_CODE=None, SCHUL_NM=None,
                         SCHUL_KND_SC_NM=None, LCTN_SC_NM=None, FOND_SC_NM=None):

        paramlist = []

        if SD_SCHUL_CODE is not None:
            SSC = f'&SD_SCHUL_CODE={SD_SCHUL_CODE}'
            paramlist.append(SSC)

        if SCHUL_NM is not None:
            SN = f'&SCHUL_NM={SCHUL_NM}'
            paramlist.append(SN)

        if SCHUL_KND_SC_NM is not None:
            SKSN = f'&SCHUL_KND_SC_NM={SCHUL_KND_SC_NM}'
            paramlist.append(SKSN)

        if LCTN_SC_NM is not None:
            LSN = f'&LCTN_SC_NM={LCTN_SC_NM}'
            paramlist.append(LSN)

        if FOND_SC_NM is not None:
            FSN = f'&FOND_SC_NM={FOND_SC_NM}'
            paramlist.append(FSN)

        query = "".join(paramlist)  # IDK

        return await self.http.schoolInfo(query)

    async def mealServiceDietInfo(self, ATPT_OFCDC_SC_CODE=None, SD_SCHUL_CODE=None,
                                  MMEAL_SC_CODE=None, MLSV_YMD=now, MLSV_FROM_YMD=None, MLSV_TO_YMD=None):
        paramlist = []

        if ATPT_OFCDC_SC_CODE is not None:
            AOSC = f'&ATPT_OFCDC_SC_CODE={ATPT_OFCDC_SC_CODE}'
            paramlist.append(AOSC)

        if SD_SCHUL_CODE is not None:
            ASC = f'&SD_SCHUL_CODE={SD_SCHUL_CODE}'
            paramlist.append(ASC)

        if MLSV_YMD is not None:
            MY = f'&MLSV_YMD={MLSV_YMD}'
            paramlist.append(MY)

        if MLSV_FROM_YMD is not None:
            MFY = f'&MLSV_FROM_YMD={MLSV_FROM_YMD}'
            paramlist.append(MFY)

        if MLSV_TO_YMD is not None:
            MTY = f'&MLSV_TO_YMD={MLSV_TO_YMD}'
            paramlist.append(MTY)

        query = "".join(paramlist)

        return await self.http.mealServiceDietInfo(query)

    async def SchoolSchedule(self, ATPT_OFCDC_SC_CODE=None, SD_SCHUL_CODE=None, DGHT_CRSE_SC_NM=None,
                             SCHUL_CRSE_SC_NM=None, AA_YMD=now, AA_FROM_YMD=None, AA_TO_YMD=None):

        paramlist = []

        if ATPT_OFCDC_SC_CODE is not None:
            AOSC = f'&ATPT_OFCDC_SC_CODE={ATPT_OFCDC_SC_CODE}'
            paramlist.append(AOSC)

        if SD_SCHUL_CODE is not None:
            ASC = f'&SD_SCHUL_CODE={SD_SCHUL_CODE}'
            paramlist.append(ASC)

        if DGHT_CRSE_SC_NM is not None:
            DCSN = f'&DGHT_CRSE_SC_NM={DGHT_CRSE_SC_NM}'
            paramlist.append(DCSN)

        if SCHUL_CRSE_SC_NM is not None:
            SCSN = f'&SCHUL_CRSE_SC_NM={SCHUL_CRSE_SC_NM}'
            paramlist.append(SCSN)

        if AA_YMD is not None:
            AY = f'&AA_YMD={AA_YMD}'
            paramlist.append(AY)

        if AA_FROM_YMD is not None:
            AFY = f'&AA_FROM_YMD={AA_FROM_YMD}'
            paramlist.append(AFY)

        if AA_TO_YMD is not None:
            ATY = f'&AA_TO_YMD={AA_TO_YMD}'
            paramlist.append(ATY)

        query = "".join(paramlist)

        return await self.http.SchoolSchedule(query)

    async def acaInsTiInfo(self, ATPT_OFCDC_SC_CODE=None, ADMST_ZONE_NM=None,
                           ACA_ASNUM=None, REALM_SC_NM=None, LE_ORD_NM=None, LE_CRSE_NM=None):

        paramlist = []

        if ATPT_OFCDC_SC_CODE is not None:
            AOSC = f'&ATPT_OFCDC_SC_CODE={ATPT_OFCDC_SC_CODE}'
            paramlist.append(AOSC)

        if ADMST_ZONE_NM is not None:
            AZN = f'&ADMST_ZONE_NM={ADMST_ZONE_NM}'
            paramlist.append(AZN)

        if ACA_ASNUM is not None:
            AA = f'ACA_ASNUM={ACA_ASNUM}'
            paramlist.append(AA)

        if REALM_SC_NM is not None:
            RSC = f'REALM_SC_NM={REALM_SC_NM}'
            paramlist.append(RSC)

        if LE_ORD_NM is not None:
            LON = f'LE_ORD_NM={LE_ORD_NM}'
            paramlist.append(LON)

        if LE_CRSE_NM is not None:
            LCN = f'LE_CRSE_NM={LE_CRSE_NM}'
            paramlist.append(LCN)

        query = "".join(paramlist)

        return await self.http.acaInsTiInfo(query)

    async def elsTimetable(self, ATPT_OFCDC_SC_CODE=None,
                           SD_SCHUL_CODE=None, AY=None, SEM=None, ALL_TI_YMD=now,
                           GRADE=None, CLASS_NM=None, PERIO=None, TI_FROM_YMD=None, TI_TO_YMD=None):

        paramlist = []

        if ATPT_OFCDC_SC_CODE is not None:
            AOSC = f'&ATPT_OFCDC_SC_CODE={ATPT_OFCDC_SC_CODE}'
            paramlist.append(AOSC)

        if SD_SCHUL_CODE is not None:
            ASC = f'&SD_SCHUL_CODE={SD_SCHUL_CODE}'
            paramlist.append(ASC)

        if AY is not None:
            AY = f'&AY={AY}'
            paramlist.append(AY)

        if SEM is not None:
            SEM = f'&SEM={SEM}'
            paramlist.append(SEM)

        if ALL_TI_YMD is not None:
            ATY = f'&ALL_TI_YMD={ALL_TI_YMD}'
            paramlist.append(ATY)

        if GRADE is not None:
            GE = f'&GRADE={GRADE}'
            paramlist.append(GE)

        if CLASS_NM is not None:
            CN = f'&CLASS_NM={CLASS_NM}'
            paramlist.append(CN)

        if PERIO is not None:
            PO = f'&PERIO={PERIO}'
            paramlist.append(PO)

        if TI_FROM_YMD is not None:
            TFY = f'&TI_FROM_YMD={TI_FROM_YMD}'
            paramlist.append(TFY)

        if TI_TO_YMD is not None:
            TTY = f'&TI_TO_YMD={TI_TO_YMD}'
            paramlist.append(TTY)

        query = "".join(paramlist)

        return await self.http.elsTimetable(query)



class Model:
    def __init__(self, data):
        self.meal = self.sort_lunchmenu(data)
        self.code = self.sort_code(data)
        self.schdule = self.sort_schedule(data)
        self.elstimetable = self.sort_elstimetable(data)

    def sort_meal(self, data):
        """
        json형식만 받아옵니다.

        json을 정리하여 급식 메뉴만을 ``str``로 반환합니다.
        """
        datalist = data['mealServiceDietInfo']
        datadict = datalist[1]['row']
        result = datadict[0]['DDISH_NM']
        linebreak = result.replace('<br/>', '\n')
        return linebreak

    def sort_code(self, data):
        """
        json형식만 받아옵니다.

        json을 정리하여 시도교육청코드,표준학교코드를 ``tuple``로 반환합니다.
        """
        datalist = data['schoolInfo']
        datalist1 = datalist[1]['row']
        SC_CODE = datalist1[0]['ATPT_OFCDC_SC_CODE']
        SD_SCHUL_CODE = datalist1[0]['SD_SCHUL_CODE']
        return(SD_SCHUL_CODE, SD_SCHUL_CODE)

    def sort_schedule(self, data):
        """
        json 형식만 받아옵니다.

        json을 정리하여 학사일정명을 ``str``로 반환합니다.
        """
        datalist = data['SchoolSchedule']
        datadict = datalist[1]['row']
        result = datadict[0]['EVENT_NM']
        return result

    def sort_elstimetable(self, data):
        """
        json형식만 받아옵니다.

        json을 정리하여 첫번째 교시부터 순서대로 ``list``로 반환합니다.
        """
        datalist = data['elsTimetable']
        datalist1 = datalist[1]['row']
        result = [f['ITRT_CNTNT'] for f in datalist1]
        return result



import aiohttp
import json
from .error import *


def status_info(response, querytype):
    try:
        datalist = response[querytype]
        headlist = datalist[0]['head']
        code = headlist[1]['RESULT']['CODE']
        message = headlist[1]['RESULT']['MESSAGE']
        return(code, message)
    except KeyError:
        code = response['RESULT']['CODE']
        message = response['RESULT']['MESSAGE']
        return(code, message)


def check_apikey(key):
    if any(key):
        pass
    else:
        raise APIKeyNotFound()


class Http:

    def __init__(self, KEY, Type, pIndex, pSize):
        try:
            check_apikey(KEY)
        except APIKeyNotFound:
            import traceback
            traceback.print_exc()
            pass
        self.requirement_query = self.requirement(KEY, Type, pIndex, pSize)

    async def request(self, method, url, query):
        base_url = 'https://open.neis.go.kr/hub/'
        URL = base_url + url + self.requirement_query + query

        async with aiohttp.ClientSession() as cs:
            async with cs.request(method, URL) as r:
                response = await r.text()
                data = json.loads(response)
                code, msg = status_info(data, url)

                if code == "INFO-000":
                    return data

                if code == "ERROR-300":
                    raise MissingRequiredValues(code, msg)
                if code == "ERROR-290":
                    raise AuthenticationKeyInvaild(code, msg)
                if code == "ERROR-310":
                    raise ServiceNotFound(code, msg)
                if code == "ERROR-333":
                    raise LocationValueTypeInvaild(code, msg)
                if code == "ERROR-336":
                    raise CannotExceed1000(code, msg)
                if code == "ERROR-337":
                    raise DailyTrafficLimit(code, msg)
                if code == "ERROR-500":
                    raise ServerError(code, msg)
                if code == "ERROR-600":
                    raise DatabaseConnectionError(code, msg)
                if code == "ERROR-601":
                    raise SQLStatementError(code, msg)
                if code == "INFO-300":
                    raise LimitUseAuthenticationkey(code, msg)
                if code == "INFO-200":
                    raise DataNotFound(code, msg)
                else:
                    raise HTTPException(code, msg)

    def requirement(self, KEY, Type, pIndex, pSize):
        apikey = f"?KEY={KEY}"
        reqtype = f"&Type={Type}"
        pindex = f"&pindex={pIndex}"
        psize = f"&pSize={pSize}"
        return(apikey + reqtype + pindex + psize)

    async def schoolInfo(self, query):
        return await self.request('get', 'schoolInfo', query)

    async def mealServiceDietInfo(self, query):
        return await self.request('get', 'mealServiceDietInfo', query)

    async def SchoolSchedule(self, query):
        return await self.request('get', 'SchoolSchedule', query)

    async def acaInsTiInfo(self, query):
        return await self.request('get', 'acaInsTiInfo', query)

    async def elsTimetable(self, query):
        return await self.request('get', 'elsTimetable', query)



class NeispyException(Exception):
    pass


class APIKeyNotFound(NeispyException):
    def __init__(self):
        super().__init__("API키를 찾을수없습니다. 샘플키로 요청합니다.")


class HTTPException(NeispyException):
    def __init__(self, code, message):
        super().__init__(f'{code} {message}')


class MissingRequiredValues(HTTPException):
    pass


class AuthenticationKeyInvaild(HTTPException):
    pass


class ServiceNotFound(HTTPException):
    pass


class LocationValueTypeInvaild(HTTPException):
    pass


class CannotExceed1000(HTTPException):
    pass


class DailyTrafficLimit(HTTPException):
    pass


class ServerError(HTTPException):
    pass


class DatabaseConnectionError(HTTPException):
    pass


class SQLStatementError(HTTPException):
    pass


class LimitUseAuthenticationkey(HTTPException):
    pass


class DataNotFound(HTTPException):
    pass
